package org.greenstone.atea;

import java.util.*;
import java.io.*;

import org.apache.log4j.Logger;

/**
* TO COMPILE OR RUN, FIRST DO:
 *    cd maori-lang-detection/apache-opennlp-1.9.1
 *    export OPENNLP_HOME=`pwd`
 *    cd maori-lang-detection/src
 *
 * TO COMPILE:
 *    maori-lang-detection/src$
 *       javac -cp ".:../conf:../lib/*" org/greenstone/atea/RandomURLsForDomainGenerator.java
 *
 * TO RUN:
 *    maori-lang-detection/src$
 *       java -cp ".:../conf:../lib/*" org/greenstone/atea/RandomURLsForDomainGenerator ../domains.txt
 *
*/
public class RandomURLsForDomainGenerator {
    static Logger logger = Logger.getLogger(org.greenstone.atea.RandomURLsForDomainGenerator.class.getName());
    
    private final MongoDBAccess mongodbAccess;
    private final int numURLs;

    //private String[] urls;
    ArrayList<String> urlsList = new ArrayList<String>();
    
    public RandomURLsForDomainGenerator(MongoDBAccess mongodbAccess,
					File domainsFile,
					int numURLs)
    {
	this.mongodbAccess = mongodbAccess;
	this.numURLs = numURLs;

	// 1. read each url from the domainsFile
	  // 1a. do the query
	  // 1b. add the arraylist result to urls

	try (
	     BufferedReader reader = new BufferedReader(new FileReader(domainsFile));
	     ) {
	    
	    
	    String domain;
	    
	    while((domain = reader.readLine()) != null) {
		domain = domain.trim();
		if(!domain.equals("")) {
		    ArrayList<String> moreURLs = mongodbAccess.queryAllMatchingIsMRIURLs(domain);
		    urlsList.addAll(moreURLs);
		}
	    }
	} catch(Exception e) {
	    logger.error("Unable to read URLs from file " + domainsFile.getAbsolutePath());
	    logger.error(e.getMessage(), e);
	}

	// copy into array
	/*
	urls = new String[urlsList.size()];
	String[] urls = urlsList.toArray(urls);
	urlsList.clear();
	*/

	/*
	// 2. generate numURLs of UNIQUE numbers between 0 to urls.length
	// https://stackoverflow.com/questions/8115722/generating-unique-random-numbers-in-java
	// https://www.geeksforgeeks.org/iterator-vs-foreach-in-java/
	// BETTER: https://stackoverflow.com/questions/5505927/how-to-generate-a-random-permutation-in-java
	
	
	// 3. then for each number, write the url at that index in array urls into file.
	*/

	// Shuffle the urlsList, then write out the first numURLs into a file.
	
	// BETTER: https://stackoverflow.com/questions/5505927/how-to-generate-a-random-permutation-in-java
	File parentFolder = domainsFile.getParentFile();
	String fileName = domainsFile.getName();
	File outFile = new File(parentFolder, "random"+numURLs+"_"+fileName);
	
	// shuffle list and take the first n
	try (
	     Writer writer = new BufferedWriter(new FileWriter(outFile));
	     ) {
	    Collections.shuffle(urlsList);
	    for (int i=0; i<numURLs; i++) {
		String url = urlsList.get(i);
		//System.out.println(list.get(i));
		writer.write(url + "\n");
	    }
	} catch(Exception e) {
	    logger.error("Unable to write to file " + outFile.getAbsolutePath());
	    logger.error(e.getMessage(), e);
	}
	
    }
	
    public static void printUsage() {
	System.err.println("RandomURLsForDomainGenerator <domains.txt> <numURLs>");
    }
    
    public static void main(String args[]) {
	if(args.length != 2) {
	    printUsage();
	    System.exit(-1);
	}

	try (
	     MongoDBAccess mongodb = new MongoDBAccess();
	     ) {
	    File domainsFile = new File(args[0]);
	    if(!domainsFile.exists()) {
		System.err.println("File " + domainsFile + " does not exist");
		System.exit(-1);
	    }

	    int genNumURLs = Integer.parseInt(args[1]);

	    mongodb.connectToDB();
	    
	    RandomURLsForDomainGenerator gen =
		new RandomURLsForDomainGenerator(mongodb, domainsFile, genNumURLs);
	    
	} catch(Exception e) {
	    logger.error(e.getMessage(), e);
	}
    }
}
